@@ -1,15 +1,19 @@
+import fileinput
 import sys
+from typing import NoReturn
 
 import click
 import deal
 
 deal.activate()
 
-from .constants import MAX_PASSWORD_LENGTH, MIN_PASSWORD_LENGTH
+from .constants import DEFAULT_BLOCKS, PASSWORD_LENGTH_MAX, PASSWORD_LENGTH_MIN
+from .engine import search
 from .exceptions import DumbValueError
-from .pwgen import search
 
 
+@deal.has("io", "global", "stderr", "stdout")
+@deal.raises(SystemExit, RuntimeError)
 @click.command(
     no_args_is_help=True,
 )
@@ -41,7 +45,7 @@ from .pwgen import search
 @click.option(
     "--blocklist",
     type=str,
-    default="""'";""",
+    default=DEFAULT_BLOCKS,
     show_default=True,
     help="Characters that may not be in the password.",
 )
@@ -54,19 +58,32 @@ from .pwgen import search
 @click.argument(
     "length",
     type=click.IntRange(
-        min=MIN_PASSWORD_LENGTH,
-        max=MAX_PASSWORD_LENGTH,
+        min=PASSWORD_LENGTH_MIN,
+        max=PASSWORD_LENGTH_MAX,
     ),
 )
+@click.option(
+    "--specials",
+    envvar="DUMBPW_SPECIALS",
+    help=(
+        "Non-alphanumeric characters that may be in the password. "
+        "Pass '-' to read from standard input."
+    ),
+    default="",
+)
 def cli(
     length: int,
     min_uppercase: int,
     min_lowercase: int,
     min_digits: int,
     min_specials: int,
+    specials: str,
     blocklist: str,
     allow_repeating: bool,
-) -> int:
+) -> NoReturn:
+    if specials == "-":
+        specials = "".join(char for char in fileinput.input(files="-")).strip()
+
     try:
         try_password = search(
             length=length,
@@ -74,17 +91,14 @@ def cli(
             min_lowercase=min_lowercase,
             min_digits=min_digits,
             min_specials=min_specials,
+            specials=specials,
             blocklist=blocklist,
             allow_repeating=allow_repeating,
         )
     except DumbValueError as ve:
-        print(ve)
-        return 1
+        print(ve, file=sys.stderr)
+        sys.exit(1)
 
     print(try_password)
 
-    return 0
-
-
-if __name__ == "__main__":
-    sys.exit(cli())
+    sys.exit(0)
